package cn.com.qmth.examcloud.exchange.inner.service.upyun;

import java.io.File;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import org.apache.commons.compress.utils.IOUtils;
import org.apache.http.HttpStatus;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import com.UpYun;
import com.upyun.UpException;
import com.upyun.UpYunUtils;

import cn.com.qmth.examcloud.commons.base.exception.ExamCloudRuntimeException;
import cn.com.qmth.examcloud.commons.base.exception.StatusException;

/**
 * upyun SDK定制版
 *
 * @author WANGWEI
 * @date 2018年11月21日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class UpYunClient {

	/**
	 * 空间名
	 */
	protected String bucketName = null;

	/**
	 * 操作员名
	 */
	protected String userName = null;

	/**
	 * 操作员密码
	 */
	protected String password = null;

	protected String md5Password = null;

	public static final String API_DOMAIN = "v0.api.upyun.com";

	private static final String MKDIR = "mkdir";

	private final String METHOD_PUT = "PUT";

	private final String DATE = "Date";

	private final String AUTHORIZATION = "Authorization";

	private CloseableHttpClient httpclient;

	private RequestConfig requestConfig;

	/**
	 * 构造函数
	 *
	 * @param bucketName
	 * @param userName
	 * @param password
	 */
	public UpYunClient(String bucketName, String userName, String password) {

		httpclient = HttpClients.custom().disableAutomaticRetries().build();;
		requestConfig = RequestConfig.custom().setConnectionRequestTimeout(10000)
				.setSocketTimeout(10000).setConnectTimeout(10000).build();

		this.bucketName = bucketName;
		this.userName = userName;
		this.password = password;
		this.md5Password = md5(password);
	}

	/**
	 * 上传文件
	 *
	 * @author WANGWEI
	 * @param filePath
	 * @param file
	 * @return
	 */
	public String writeFile(String filePath, File file) {

		UpYun upYun = new UpYun(bucketName, userName, password);
		try {
			upYun.writeFile(filePath, file, true);
		} catch (Exception e) {
			throw new StatusException("EX-100001", "upyun upload failure");
		}

		String path = "	http://" + bucketName + ".b0.upaiyun.com" + filePath;
		return path;
	}

	/**
	 * 上传文件
	 *
	 * @author WANGWEI
	 * @param filePath
	 * @param in
	 * @return
	 */
	public String writeFile(String filePath, InputStream in) {

		String url = "http://" + API_DOMAIN + filePath;

		HttpPut httpPut = new HttpPut(url);
		httpPut.setConfig(this.requestConfig);

		try {
			String date = getGMTDate();
			httpPut.addHeader(DATE, date);

			httpPut.addHeader(AUTHORIZATION, UpYunUtils
					.sign(METHOD_PUT, date, filePath, userName, this.md5Password, null).trim());

			httpPut.addHeader(MKDIR, "true");

			httpPut.setEntity(new InputStreamEntity(in));
			CloseableHttpResponse response = httpclient.execute(httpPut);
			int statusCode = response.getStatusLine().getStatusCode();
			System.out.println(EntityUtils.toString(response.getEntity(), "UTF-8"));
			if (HttpStatus.SC_OK != statusCode) {
				throw new StatusException("EX-100001", "upyun upload failure");
			}
		} catch (StatusException e) {
			throw e;
		} catch (UpException e) {
			throw new ExamCloudRuntimeException(e);
		} catch (Exception e) {
			throw new ExamCloudRuntimeException(e);
		} finally {
			httpPut.releaseConnection();
			IOUtils.closeQuietly(in);
		}

		String path = "	http://" + bucketName + ".b0.upaiyun.com" + filePath;
		return path;

	}

	/**
	 * 对字符串进行 MD5 加密
	 *
	 * @param str
	 *            待加密字符串
	 * @return 加密后字符串
	 */
	private static String md5(String str) {
		char hexDigits[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd',
				'e', 'f'};
		MessageDigest md5 = null;
		try {
			md5 = MessageDigest.getInstance("MD5");
			md5.update(str.getBytes("UTF-8"));
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}
		byte[] encodedValue = md5.digest();
		int j = encodedValue.length;
		char finalValue[] = new char[j * 2];
		int k = 0;
		for (int i = 0; i < j; i++) {
			byte encoded = encodedValue[i];
			finalValue[k++] = hexDigits[encoded >> 4 & 0xf];
			finalValue[k++] = hexDigits[encoded & 0xf];
		}

		return new String(finalValue);
	}

	/**
	 * 获取 GMT 格式时间戳
	 *
	 * @return GMT 格式时间戳
	 */
	private String getGMTDate() {
		SimpleDateFormat formater = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss 'GMT'",
				Locale.US);
		formater.setTimeZone(TimeZone.getTimeZone("GMT"));
		return formater.format(new Date());
	}

}
